@@ -3,5 +3,8 @@ from . import opt
 from . import blm
 from . import misc
 from . import search
+from . import predictor
 from .predictor import base_predictor
 from .variable import variable
+
+__version__ = "1.0.0"
