@@ -1,7 +1,7 @@
-from ..predictor import base_predictor
+import physbo.predictor
 
 
-class predictor(base_predictor):
+class predictor(physbo.predictor.base_predictor):
     """Predictor using Baysean linear model
 
     Attributes
@@ -22,7 +22,7 @@ class predictor(base_predictor):
 
         See also
         ========
-        physbo.predictor.base_predictor
+        physbo.base_predictor
         """
         super(predictor, self).__init__(config, model)
         self.blm = None
@@ -214,17 +214,15 @@ class predictor(base_predictor):
 
         if N == 1:
             if test.Z is None:
-                try:
-                    test.X.shape[1]
-                    self.blm.update_stats(test.X[0, :], test.t)
-                except:
+                if test.X.ndim == 1:
                     self.blm.update_stats(test.X, test.t)
+                else:
+                    self.blm.update_stats(test.X[0, :], test.t)
             else:
-                try:
-                    test.Z.shape[1]
-                    self.blm.update_stats(test.X[0, :], test.t, psi=test.Z[0, :])
-                except:
+                if test.Z.ndim == 1:
                     self.blm.update_stats(test.X, test.t, psi=test.Z)
+                else:
+                    self.blm.update_stats(test.X[0, :], test.t, psi=test.Z[0, :])
         else:
             for n in range(N):
                 if test.Z is None:
