@@ -1,12 +1,7 @@
-import numpy as np
-from . import cov
-from . import lik
-from . import mean
-from . import core
-from ..predictor import base_predictor
+import physbo.predictor
 
 
-class predictor(base_predictor):
+class predictor(physbo.predictor.base_predictor):
     """ predictor """
 
     def __init__(self, config, model=None):
@@ -64,6 +59,9 @@ class predictor(base_predictor):
         """
         pass
 
+    def update(self, training, test):
+        self.prepare(training)
+
     def prepare(self, training):
         """
         Initializing model by using training data set
