@@ -2,14 +2,14 @@ import numpy as np
 import scipy
 
 
-def gauss_elim(L, t):
+def gauss_elim(U, t):
     """
     Calculate alpha using scipy.linalg.solve_triangular.
-    alpha = (L^T L)^-1 t = L^-1 [(L^T)-1 t]
+    alpha = (U^T U)^-1 t = U^-1 [(U^T)-1 t]
 
     Parameters
     ----------
-    L: (M, M) array_like
+    U: (M, M) array_like
     A triangular matrix
     t: (M,) or (M, N) array_like
 
@@ -19,10 +19,10 @@ def gauss_elim(L, t):
     Solution to the system L^T alpha = t. Shape of return matches t.
     """
     alpha = scipy.linalg.solve_triangular(
-        L.transpose(), t, lower=True, overwrite_b=False, check_finite=False
+        U.transpose(), t, lower=True, overwrite_b=False, check_finite=False
     )
 
     alpha = scipy.linalg.solve_triangular(
-        L, alpha, lower=False, overwrite_b=False, check_finite=False
+        U, alpha, lower=False, overwrite_b=False, check_finite=False
     )
     return alpha
